# Generated by Django 3.2.13 on 2022-08-30 18:14

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Ticket',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('semaphore_name', models.CharField(help_text='The name of the semaphore. Create as many rows as you want with the same name; the semaphore by that name will hold that many tickets.', max_length=255)),
            ],
        ),
    ]
